@@ -1,3 +1,14 @@
+/*
+ * Copyright European Organization for Nuclear Research (CERN)
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * You may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Authors:
+ * - Muhammad Aditya Hilmy, <mhilmy@hey.com>, 2020
+ */
+
 export interface Instance {
   displayName: string;
   name: string;
@@ -17,9 +28,13 @@ export interface RucioX509Auth extends RucioAuth {
   key?: string;
 }
 
-export type RucioAuthCredentials = RucioUserpassAuth | RucioX509Auth;
+export interface RucioX509ProxyAuth extends RucioAuth {
+  proxy: string;
+}
 
-export type RucioAuthType = 'userpass' | 'x509';
+export type RucioAuthCredentials = RucioUserpassAuth | RucioX509Auth | RucioX509ProxyAuth;
+
+export type RucioAuthType = 'userpass' | 'x509' | 'x509_proxy';
 
 export interface InstanceConfig {
   activeInstance?: string;
